/* @(#)field_minmax.c	1.3 11/26/97
Copyright 2002 AOI Industries, Inc. All rights reserved.

License is granted to copy, to use, and to make and to use derivative
works for any purpose, provided that the copyright notice and this
license notice is included in all copies and any derivatives works and
in all  related documentation.  Licensor grants no other licenses expressed 
or implied and the licensee acknowleges that Licensor have no liability for 
licensee's use or for any derivative works made by licensee. 

This software is provided AS IS.

Licensor disclaim and licensee agrees that all warranties, express or
implied, including without limitation the implied warranties of
merchantability and fitness for a particular purpose.  Notwithstanding
any other provision contained herein, any liability for damages
resulting from the software or its use is expressly disclaimed,
including consequential or any other indirect damages, whether arising
in contract, tort (including negligence) or strict liability, even if
Licensor is advised of the possibility of such damages.  
 */
/* Determine the min/max value that can be contained by the field.  These
 * routines are intended to predict the largest/smallest value that 'sprintf'
 * will place in a field with width 'width'.
 *
 * Exceptions:  The computation for 'minflt/maxflt' breaks down when the
 *              precision is larger than the width.
 */
#include <math.h>
#include <limits.h>
#include <formdef.h>

long field_minint(int width)
{
    double x;
    /* Only need to use field width */
    x = -(pow(10.0, (double)(width-1)) - 1);

    if (x < INT_MIN) return INT_MIN;
    else return x;
}

long field_maxint(int width)
{
    double x;
    x = pow(10.0, (double)width) - 1;

    if (x > INT_MAX) return INT_MAX;
    else return x;
}

double field_minflt(int width, int precision)
{
    register double x;
    register int n;

    n = precision + 2;
    if (n + 1 > width) return 0.0;

    /* Need both field width and precision */
    if (precision) {
    	x = pow(10.0, (double)(width - n));
    	x -= pow(10.0, (double)(-(precision)));
    } else {
	x = pow(10.0, (double)(width - 1)) - 1;
    }

    return -x;
}

double field_maxflt(int width, int precision)
{
    register double x;
    register int n;

    n = precision + 1;
    if (n + 1 > width) return 0.0;

    /* Need both field width and precision */
    if (precision) {
    	x = pow(10.0, (double)(width - n));
    	x -= pow(10.0, (double)(-(precision)));
    } else {
	x = pow(10.0, (double)width) - 1;
    }

    return x;
}
