/* @(#)form.h	1.41 09/13/02
 
Copyright 2002 AOI Industries, Inc. All rights reserved.

License is granted to copy, to use, and to make and to use derivative
works for any purpose, provided that the copyright notice and this
license notice is included in all copies and any derivatives works and
in all  related documentation.  Licensor grants no other licenses expressed 
or implied and the licensee acknowleges that Licensor have no liability for 
licensee's use or for any derivative works made by licensee. 

This software is provided AS IS.

Licensor disclaim and licensee agrees that all warranties, express or
implied, including without limitation the implied warranties of
merchantability and fitness for a particular purpose.  Notwithstanding
any other provision contained herein, any liability for damages
resulting from the software or its use is expressly disclaimed,
including consequential or any other indirect damages, whether arising
in contract, tort (including negligence) or strict liability, even if
Licensor is advised of the possibility of such damages.  
 */
/* Terminal Specific Control Sequences For Forms/Menu Routines.
 */

#ifndef FORM_H
#define FORM_H

#include <strings.h>
#include <limits.h>
#include <float.h>
#ifdef __linux__
#include <termcap.h>
#endif

#if defined(__sun__)
char *tgoto(char *cm, int destcol, int destline);
#endif

/* Move cursor with 1,1 being the upper left corner */
#define putcursor(row, col) writeterm(tgoto(fm_t.cm, col-1, row-1))

#define clr_screen()    writeterm(fm_t.cl)	/* clear entire screen */
#ifdef clr_eol
#undef clr_eol
#endif
#define clr_eol()       writeterm(fm_t.ce)	/* clear to end of line */
#ifdef clr_eos
#undef clr_eos
#endif
#define clr_eos()	writeterm(fm_t.cd)	/* clear to end of screen */
#define del_line()	writeterm(fm_t.dl)	/* delete current line */
#define cursor_off()    writeterm(fm_t.CE)      /* turn off cursor */
#define cursor_on()     writeterm(fm_t.CS)      /* turn cursor on */
#define dwide_on()	writeterm(fm_t.WS)	/* start double wide */
#define dwide_off()	writeterm(fm_t.WE)	/* end double wide */
#define normal_video()	writeterm(fm_t.me)	/* return to normal video */
#define rev_on()	writeterm(fm_t.mr)	/* start reverse video */
#define rev_off()	normal_video()		/* end reverse video */
#define bold_on()	writeterm(fm_t.md)	/* start bold */
#define bold_off()	normal_video()		/* end bold */
#define blink_on()	writeterm(fm_t.mb)	/* start blink */
#define blink_off()	normal_video()		/* end blink */
#define underline_on()  writeterm(fm_t.us)	/* start underline */
#define underline_off()	normal_video()		/* end underline */

struct TERMTYPE {
	char *cl;	/* Clear Screen */
	char *cd;	/* Clear to end of screen */
	char *ce;	/* Clear to end of line */
	char *cm;	/* Cursor Motion */
	char *kd;	/* sent by down arrow */
	char *kl;	/* sent by left arrow */
	char *kr;	/* sent by right arrow */
	char *ku;	/* sent by up arrow */
	char *k1;	/* sent by pf1 */
	char *k2;	/* sent by pf2 */
	char *k3;	/* sent by pf3 */
	char *k4;	/* sent by pf4 */
	char *kh;	/* sent by home key */
        char *kc;       /* sent by clear messages key */
	char *mb;	/* mode blink */
	char *md;	/* mode bold */
	char *mr;	/* mode reverse */
	char *me;	/* return to normal mode */
	char *nd;	/* non-destructive space */
	char *se;	/* end standout mode */
	char *so;	/* start standout mode */
	char *ue;	/* end underscore mode */
	char *us;	/* start underscore mode */
	char *dl;	/* delete line */
	char *CE;	/* cursor off */
	char *CS;	/* cursor on */
	char *WE;	/* double wide off */
	char *WS;	/* double wide on */
	char *kN;       /* next page */
	char *kP;       /* previous page */
	char *help;     /* help key */
	char tbuf[1024];
};

extern struct TERMTYPE fm_t;

#define BEL	'\07'	/* ring bell */
#define BS	'\010'	/* last field (also up arrow) */
#define TAB	'\011'	/* next field (also down arrow */
#define DEL	'\177'	/* destructive backspace */
#define CAN	'\05'	/* ^E, cancel form/menu */
#define CTRL_U	'\025'  /* ^U, clear field */
#define CTRL_L	'\014'  /* ^L, refresh screen */
#define SPACE   '\040'  /* SPACE bar */

/* Bit definitions for initform 'mode' argument */
#define TTY_RAW_MODE   1
#define TTY_BLOCK_MODE 2

/* Values returned by 'getKchar' for keystrokes generating sequences.
 */

#define UP_ARROW    0x100
#define DOWN_ARROW  0x101
#define RIGHT_ARROW 0x102
#define LEFT_ARROW  0x103
#define PF1         0x104
#define PF2         0x105
#define PF3         0x106
#define PF4         0x107
#define J1          0x108   /* joystick button */
#define HOME        0x109
#define PREV_SCRN   0x10A
#define NEXT_SCRN   0x10B
#define HELP        0x10C
#define CLEAR_MSG   0x10D

/* Values returned for non-required integer/float fields with defaults 
   that the user has left blank.
 */
#define BLANK_INT    INT_MIN 
#define BLANK_LONG   INT_MIN 
#define BLANK_FLOAT  -FLT_MAX
#define BLANK_DOUBLE -FLT_MAX

/* Options constants */
#define FIELDONLY 2         /* Only display fields (w/o clearing screen) */
#define CLRSCREEN 1	    /* clear the screen on entry */
#define NOCLRSCREEN 0	    /* don't clear the screen on entry */
#define CAN_ESCAPE 0	    /* user can escape this form */
#define NOESCAPE 1	    /* user can't escape this form */

/* putform return codes */
#define formBARCODE  3	    /* data for barcode field available */
#define formPREV     2	    /* for single field forms, BS was pressed exit */
#define formNEXT     1	    /* for single field forms, TAB was pressed exit */
#define formNONE     0	    /* normal return */
#define formCANCEL  -1	    /* ^E, cancel was pressed */
#define formTIMEOUT -2	    /* pc-based timeout */
#define formHOME    -3	    /* ^T, pop to top key pressed */

/*
** ANSI funtion prototypes
*/
void initform(int raw);
void writeterm(char *str);
int  putform(int clrscr, int nocancel, int field, const char *form_key,...);
int  putmenu(int clrscr, int nocancel, int max, const char *menu_key, ...);
int  getKchar(void);
int  getKchar_cnt(void);
int  getKchar_msgs(int row, int col);
int  ifchar_msgs(int row, int col);
void ungetKchar(int c);
void logError(int, const char *fmt, ...);
void to_caps(char*);
void truncate_leading(char *);
void truncate_trailing(char *);
void blankpad(char *str, int len);
void uiSetNorm(void);
#endif
