/* @(#)formdef.h	1.18 10/29/02

Copyright 2002 AOI Industries, Inc. All rights reserved.

License is granted to copy, to use, and to make and to use derivative
works for any purpose, provided that the copyright notice and this
license notice is included in all copies and any derivatives works and
in all  related documentation.  Licensor grants no other licenses expressed 
or implied and the licensee acknowleges that Licensor have no liability for 
licensee's use or for any derivative works made by licensee. 

This software is provided AS IS.

Licensor disclaim and licensee agrees that all warranties, express or
implied, including without limitation the implied warranties of
merchantability and fitness for a particular purpose.  Notwithstanding
any other provision contained herein, any liability for damages
resulting from the software or its use is expressly disclaimed,
including consequential or any other indirect damages, whether arising
in contract, tort (including negligence) or strict liability, even if
Licensor is advised of the possibility of such damages.  
 */

#ifndef FORMDEF_H
#define FORMDEF_H

#include <stdarg.h> /* for fieldef function prototype */

#define TRUE 1
#define FALSE 0

    /* when adding new symbols to the form grammar be sure to
     * update the following #defines.  Failure to do so could 
     * cause false error messages about an illegal form. */
#define RANGE_SYM          ".<>[]{}()"
#define DATATYPE_SYM       "sdlxfFXYb[]{}()"
#define RANGE_DATATYPE_SYM "<>[]{}()sdlxfFXYb"
#define POSITION_SYM       "%@;+-"
#define FIELD_ATTR_SYM     "*!#?"
#define SCREEN_ATTR_SYM    "^~_"

#define display_units(v) ((formunits == MILLIMETERS) \
			  ? micron2mm(v):micron2inch(v))

#define to_internal_units(v) ((formunits == MILLIMETERS) \
			      ? mm2micron(v):inch2micron(v))

/* Forms management internal data structures. */

/*
 * Used by scan_form to determine what action definitions to ignore
 * Note:  This probably doesn't need to be a bit field, it could just be
 * two parameters to scan_form.
 */

typedef struct {
    unsigned int NoPosition : 1;     /* getYN forms can't have positions */
    unsigned int NoInputFields : 1;  /* menus/getYN can't have input fields */
} InvalidActions;


/* Bit assignments for character attribute modes */
#define BLINK          1
#define BOLD           2
#define REVERSE        4
#define UNDERLINE      8
#define DOUBLE_WIDE 0x10

/* field choice list record */
struct CHOICEREC {
    char *choice;   	    	/* string to display */
    int   ord;	    	    	/* ordinal value to return */
    int   cand;	    	    	/* mark if candidate selection */
    struct CHOICEREC *next; 	/* forward link */
    struct CHOICEREC *prev; 	/* backward link */
};


/* type of fields */
enum DTYPE { None, Char, Int, Long, Float, Double, Hex, 
	     List, Dir, Str };


    /* individual field record */
struct FIELDREC {
    struct FIELDREC *flink;	/* forward link */
    struct FIELDREC *blink;	/* back link */
    int row, col;		/* location of field */
    int ordinal;		/* ordinal position of field */
    int fillc;			/* character to use for 'blank' in field */
    int required;		/* field is required */
    int have_entry;		/* entry was made */
    int echo;			/* do not echo input */
    int defval;			/* default value available */
    int width;			/* field width */
    int precision;		/* precision of float (for default display) */
    enum DTYPE dtype;		/* data type of field */
    long dmin, dmax;		/* min, max for integers */
    double fmin, fmax;		/* min, max for floating point */
    void *argp;			/* ptr to supplied argument */
    int smode;			/* screen mode for field */
    int choice_displayed;	/* true if full choice displayed */
    int num_choices;    	/* num of entries in choice list */
    struct CHOICEREC *choice_list; /* list of choices for List field */
    struct CHOICEREC *curr_choice; /* current position in choice_list */
    char file_ext[12];		/* optional file extension for dir fields */
    char fdata[4];		/* characters entered - variable length; MUST
				   be last member of structure */
};


    /* list of all fields in a form */
struct FIELD_LIST {
    struct FIELDREC *head;
    struct FIELDREC *tail;
};

    /* Return values from 'getfspec' */
enum FSPECS { LASTSPEC, NOARGSPEC, COORDMAX, COORDMIN, INTMAX, INTMIN,
	      LONGMAX, LONGMIN, FLOATMAX, FLOATMIN, DOUBLEMAX, DOUBLEMIN };

/* ansi prototypes for internal form functoins */
void    fbadchar (struct FIELDREC *cf, int i);
void    fbadlist (struct FIELDREC *cf);
char *  fieldef(struct FIELD_LIST *fldlist, char *fp, struct FIELDREC **fldrec,
                int row, int col, int scrmode, va_list *ap);
void    fldrelease(struct FIELD_LIST *fldlist);
int 	forminput(struct FIELD_LIST *fldlist, int nocancel, int field);
int     frmPeekAhead(char **p, char *lookfor, char *before);
char *  getfspec(char *fp, char *sp, struct FIELDREC *fr, int *, int *,
		 enum FSPECS *);
int     listlen(char *fp);
char *  next_choice(char **fp, int *len);
char *  readonly(char *fp, int scrmode, int *col, va_list *ap);
void    reset_attr(int smode);
void    scan_form(const char *fp, int clrscr, int *scrmode, int *row,
                  int *col,struct FIELD_LIST *fld_list,
		  InvalidActions v, va_list *ap);
int     valid_field(struct FIELDREC *cf, int ret);
long 	field_minint(int);
long 	field_maxint(int);
double 	field_minflt(int, int);
double 	field_maxflt(int, int);

#endif /* FORMDEF_H */
