/* @(#)forminput.c	1.45 10/29/02
Copyright 2002 AOI Industries, Inc. All rights reserved.

License is granted to copy, to use, and to make and to use derivative
works for any purpose, provided that the copyright notice and this
license notice is included in all copies and any derivatives works and
in all  related documentation.  Licensor grants no other licenses expressed 
or implied and the licensee acknowleges that Licensor have no liability for 
licensee's use or for any derivative works made by licensee. 

This software is provided AS IS.

Licensor disclaim and licensee agrees that all warranties, express or
implied, including without limitation the implied warranties of
merchantability and fitness for a particular purpose.  Notwithstanding
any other provision contained herein, any liability for damages
resulting from the software or its use is expressly disclaimed,
including consequential or any other indirect damages, whether arising
in contract, tort (including negligence) or strict liability, even if
Licensor is advised of the possibility of such damages.  
 */

/* Handle keyboard input for the form.  The screen has been painted, and the
   field list created.  We place the cursor at the first field, and process
   keystokes as they come in.
 */
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>

#include "form.h"
#include "formdef.h"

/* Data global to these routines */
extern int default_language;

/* Do final field validation.  Return 0 if ok, -1 if user needs to make a
   change.
 */
static
struct FIELDREC *
form_final_validate(struct FIELD_LIST *fldlist)
{
    int i;
    struct FIELDREC *cf;

    cf = fldlist->head;
    while (cf) {
	if (!valid_field(cf, TRUE)) {
	    if (cf->dtype == List || cf->dtype == Dir || cf->dtype == Str) {
		memset(cf->fdata, '\0', cf->width + 1);
		putcursor(cf->row, cf->col);
		for (i = 0; i < cf->width; i++) putchar(cf->fillc);
		break;
	    }
	    putcursor(cf->row, cf->col);
	    break;
	}

	if (cf->required && !cf->have_entry) {
	    reset_attr(0);
	    if (default_language)
		logError(0, "Field can not be blank!!\7", 0);
	    else
		logError(0, "Field can not be blank!!\7", 0);

	    reset_attr(cf->smode);
	    putcursor(cf->row, cf->col);
	    break;
	}
	cf = cf->flink;

    }	/* While still fields in list */

    return cf;
}

/* ------------------------------------------------------------------------- */

/* islistentry()
 *
 * Return TRUE if current character appended with string already in
 * fdata forms a valid case-insensitive prefix for one of the list
 * entries.  If the prefix is unique set unique to TRUE and return
 * the corresponding display string; otherwise set to FALSE.
 */

static int 
islistentry(struct FIELDREC *cf, char c, int pos,
	    int *unique, struct CHOICEREC **cr)
{
    struct CHOICEREC *tmpcr;
    int i, len, cand;
    char *p, *q, tmp[64];

    memset(tmp, '\0', sizeof(tmp));

    p = cf->fdata;
    q = tmp;
    len = 0;
    while (*p && (len < pos)) {
	*q++ = *p++;
	len++;
    }
    tmp[len] = c;	/* append typed in character */
    tmp[len + 1] = '\0';

    if (cf->dtype == List || cf->dtype == Str)
	to_caps(tmp);	/* only work in uppercase */

    /* mark all possible candidate list entries. */
    for (i = 0, tmpcr = cf->choice_list, cand = 0;
	 i < cf->num_choices; i++, tmpcr = tmpcr->next) {

	p = strstr(tmpcr->choice, tmp);
	if (p && p == tmpcr->choice) {
	    cand++;
	    tmpcr->cand = TRUE;
	} else
	    tmpcr->cand = FALSE;
    }

    if (cand == 0) {	/* nothing found */
	*unique = FALSE;
	return (FALSE);
    } else if (cand == 1) {	/* found it */
	*unique = TRUE;
	tmpcr = cf->choice_list;
	if (!tmpcr) return TRUE;

	do {
	    if (tmpcr->cand) {
		*cr = tmpcr;
		tmpcr->cand = FALSE;
		break;
	    }
	    tmpcr = tmpcr->next;
	} while (tmpcr != cf->choice_list);

	return (TRUE);
    } else {	/* found more than one candidate */
	for (i = 0, tmpcr = cf->choice_list;
	     i < cf->num_choices; i++, tmpcr = tmpcr->next) {
	    tmpcr->cand = FALSE;
	}
	*unique = FALSE;

	if (cf->choice_displayed) {
	    memset(cf->fdata, '\0', cf->width + 1);
	    cf->fdata[0] = c;
	} else {
	    p = cf->fdata;
	    while (*p && *p == cf->fillc)	/* ignore leading spaces */
		p++;
	    while (*p && *p != cf->fillc)	/* skip until start of
						 * spaces */
		p++;
	    *p++ = c;
	    *p = '\0';
	}
	return (TRUE);
    }
}

static void 
fieldstring(struct FIELDREC *cf, char *s)
{
    int i;

    strcpy(cf->fdata, s);

    putcursor(cf->row, cf->col);
    fputs(s, stdout);
    for (i = strlen(s); i < cf->width; i++) putchar(cf->fillc);
    putcursor(cf->row, cf->col);
}

/*
 * Write the character on the screen, and place it in the field buffer.
 * Update the cursor position.  When the character is written to the last
 * field position, the cursor is placed back over it.
 */

static int
fieldchar(struct FIELDREC *cf, int c, int col)
{
    if (cf->echo) {
	if (c == ' ')
	    putchar(cf->fillc);
	else
	    putchar(c);
    }

    cf->fdata[col - cf->col] = c;

    if ((col - cf->col + 1) < cf->width)
	col++;
    else if (cf->echo) putchar(BS);

    return col;
}


/* ------------------------------------------------------------------------- */

/*
 * Set up for next/last field.  The current field is checked for proper syntax,
 * and interpreted.  The value is copied to the caller's argument pointer.
 * The cursor is placed at the start of the next/last field.
 */

static struct FIELDREC *
next_field(struct FIELD_LIST *fldlist, struct FIELDREC *cf, int *row, int *col)
{
    struct FIELDREC *start;

    if (!valid_field(cf, FALSE)) {
	putcursor(cf->row, cf->col);
	*row = cf->row;
	*col = cf->col;
	return cf;
    }

    start = cf;
    if (!(cf = cf->flink)) cf = fldlist->head;

    putcursor(cf->row, cf->col);
    *row = cf->row;
    *col = cf->col;

    reset_attr(cf->smode);

    return cf;
}


static struct FIELDREC *
prev_field(struct FIELD_LIST *fldlist, struct FIELDREC *cf, int *row, int *col)
{
    struct FIELDREC *start;

    if (!valid_field(cf, FALSE)) {
	putcursor(cf->row, cf->col);
	*row = cf->row;
	*col = cf->col;
	return cf; 
    }

    start = cf;
    if (!(cf = cf->blink)) cf = fldlist->tail;

    putcursor(cf->row, cf->col);
    *row = cf->row;
    *col = cf->col;

    reset_attr(cf->smode);

    return cf;
}

/* ------------------------------------------------------------------------- */

int
forminput(struct FIELD_LIST *fldlist, int nocancel, int field)
{
    register int c, s, i;
    int single_field, row, col;
    int unique = FALSE;
    struct FIELDREC *cf;	/* current field record pointer */

    /*
     * Scan list of fields for field corresponding to argument 'field' for
     * initial cursor placement.
     */
    cf = fldlist->head;
    if (!cf || field <= 0)
	return formNONE;	/* Return immediately if no fields or user
				 * said to position on non-existent field */

    single_field = !(cf->flink);    /* set TRUE if only one field in form */

    cf = fldlist->head;

    putcursor(cf->row, cf->col);	/* cursor to first char of field */
    row = cf->row;
    col = cf->col;
    reset_attr(cf->smode);

    for (;;) {

	/*
	 * turn attributes off in case info messages are in message
	 * queue
	 */
	reset_attr(0);
	c = getKchar();
	reset_attr(cf->smode);

	switch (c) {
	  case CAN:	/* Operator cancel */
	    if (nocancel) {
		putchar(BEL);
		break;
	    }
	    putcursor(1, 1);	/* clear any message */
	    clr_eol();
	    return (formCANCEL);

	  case HOME:	/* Operator pop to top */
	    if (nocancel) {
		putchar(BEL);
		break;
	    }
	    putcursor(1, 1);	/* clear any message */
	    clr_eol();
	    return (formHOME);

	  case '\n':
	  case '\r':	/* All done - cleanup and get out */
	    cf = form_final_validate(fldlist);
	    if (!cf) return formNONE;
	    row = cf->row;
	    col = cf->col;
	    break;

	  case BS:	
	  case DEL:	/* Delete key - backspace and erase */
	    if (col == cf->col) {
		putchar(BEL);	/* Can't delete past left bndry */
	    } else {

		/*
		 * If at last character of field, and a non-blank character
		 * is there, delete it and leave the cursor there.
		 * Otherwise, delete to the left.
		 */
		if (!(col == cf->col + cf->width - 1
		      && cf->fdata[col - cf->col] != ' ')) {

		    col--;
		    if (cf->echo)
			putchar(BS);
		}
		cf->fdata[col - cf->col] = ' ';
		if (cf->echo) {
		    putchar(cf->fillc);
		    putchar(BS);
		}
	    }
	    if (cf->dtype == List || cf->dtype == Dir || cf->dtype == Str)
		cf->choice_displayed = FALSE;
	    break;

	  case TAB:	/* Next field */
    	    /* for single field forms, TAB is a form completion character */
    	    if (single_field) {
	    	cf = form_final_validate(fldlist);
		if (!cf) return formNEXT;
		
		row = cf->row;
		col = cf->col;
		break;
	    }
	    cf = next_field(fldlist, cf, &row, &col);
	    break;

	  case DOWN_ARROW:
	    cf = next_field(fldlist, cf, &row, &col);
	    break;

	  case UP_ARROW:
	    cf = prev_field(fldlist, cf, &row, &col);
	    break;

	  case LEFT_ARROW:
	    if (col == cf->col)
		cf = prev_field(fldlist, cf, &row, &col);
	    else {
		putchar(BS);
		col--;
	    }
	    break;

	  case RIGHT_ARROW:
	    if (col == cf->col + cf->width - 1)
		cf = next_field(fldlist, cf, &row, &col);
	    else {
		writeterm(fm_t.nd);
		col++;
	    }
	    break;

	  case CTRL_U:	/* clear field */
	    putcursor(cf->row, cf->col);
	    if (cf->dtype == List || cf->dtype == Dir || cf->dtype == Str) {
		cf->choice_displayed = FALSE;
		c = cf->fillc;
	    } else
		c = ' ';

	    for (s = 0; s < cf->width; s++) {
		putchar(cf->fillc);
		cf->fdata[s] = c;
	    }
	    col = cf->col;
	    putcursor(cf->row, cf->col);
	    break;

	    /* No break here - drop thru to handle 'm' as char */
	  default:

	    /*
	     * Otherwise, if it's a printable character and corresponds to
	     * the field data type, write it out, keeping track of cursor
	     * position.
	     */
	    if (!isprint(c)) break;

	    switch (cf->dtype) {
	      case None: break;	/* Needed to satisfy ANSI warnings */

	      case Str:
	      case Dir:
	      case List:
		if (islistentry(cf, c, col - cf->col,
				&unique, &cf->curr_choice)) {

                    /* the character is part of a list entry */
		    if (unique) {
			/* the character provides a unique match for
			   a list entry 
			*/
			fieldstring(cf, cf->curr_choice->choice);

			/* 'Str' fields allow more typing, so new choice
			   can begin with same initial string as existing
			   choice.
			*/
			if (cf->dtype == List) {
			    row = cf->row;
			    col = cf->col;
			    cf->choice_displayed = TRUE;
			} else {
			    if (col - cf->col + 1 < cf->width) col++;
			    putcursor(row, col);
			    cf->choice_displayed = FALSE;
			}
		    } else {	/* the character is part of a list entry,
				 * but we do not yet know which entry */
			if (cf->choice_displayed) {
			    putcursor(cf->row, cf->col);
			    for (i=0; i < cf->width; i++)
				putchar(cf->fillc);

			    putcursor(cf->row, cf->col);
			    row = cf->row;
			    col = cf->col;
			}

			if (cf->dtype == Dir)
			    col = fieldchar(cf, c, col);
			else
			    col = fieldchar(cf, toupper(c), col);

			cf->choice_displayed = FALSE;
		    }
		} else if ((c == SPACE)
			   && ((cf->dtype != Str) || (col == cf->col)
			       || cf->fdata[col - cf->col] == ' ')) {
		    /* space character did not fit in at the current
		       location for any choice list - toggle to next entry
		    */
		    cf->curr_choice = cf->curr_choice->next;
		    fieldstring(cf, cf->curr_choice->choice);
		    row = cf->row;
		    col = cf->col;
		    cf->choice_displayed = TRUE;
		} else if (cf->dtype == Str) {
		    c = toupper(c);
		    if (cf->fdata[col - cf->col] != c) {
			for (i=col - cf->col;
			     cf->fdata[i] != ' ' && i<cf->width; i++) {
			    
			    cf->fdata[i] = ' ';
			    putchar(' ');
			}
			putcursor(row, col);
		    }
			
		    col = fieldchar(cf, c, col);
		    cf->choice_displayed = FALSE;
		} else {
		    fbadlist(cf);
		    row = cf->row;
		    col = cf->col;
		}
		break;

	      case Char:
		col = fieldchar(cf, c, col);
		break;

	      case Int:
	      case Long:
		if (isdigit(c) || c == '+' || c == '-' || c == ' ')
		    col = fieldchar(cf, c, col);
		else
		    fbadchar(cf, col - cf->col);
		break;

	      case Hex:
		if (isxdigit(c) || c == ' ')
		    col = fieldchar(cf, c, col);
		else
		    fbadchar(cf, col - cf->col);
		break;

	      case Double:
	      case Float:
		if (isdigit(c) || c == '.' || c == '-' ||
		    c == '+' || c == ' ')
		    col = fieldchar(cf, c, col);
		else
		    fbadchar(cf, col - cf->col);
		break;
	    }
	}	/* switch some char */
    }
}

/* ------------------------------------------------------------------------- */

/* Release all the memory acquired for field records */
void 
fldrelease(struct FIELD_LIST *fldlist)
{
    struct FIELDREC *fr, *fq;
    struct CHOICEREC *cr, *cq;
    int i;

    fr = fldlist->head;
    while (fr) {
	for (i = 0, cr = fr->choice_list; i < fr->num_choices; i++) {
	    free(cr->choice);
	    cq = cr->next;
	    free((char *) cr);
	    cr = cq;
	}
	fq = fr->flink;
	free((char *) fr);
	fr = fq;
    }

    fldlist->head = NULL;
    fldlist->tail = NULL;

    /* Set screen attribute back to normal */
    reset_attr(0);
}

/* ------------------------------------------------------------------------- */

/* Pick the right info message, and display it */
void 
fbadchar(struct FIELDREC *cf, int i)
{
    char *key = NULL;

    if (default_language) {
	switch (cf->dtype) {
	  case None: break;	/* Needed to satisfy ANSI Warnings */

	  case Char:
	    key = "Illegal character - Alphanumeric field!!\7";
	    break;
	  case Int:
	  case Long:
	    key = "Illegal character - Integer field!!\7";
	    break;
	  case Double:
	  case Float:
	    key = "Illegal character - Real number field!!\7";
	    break;
	  case Hex:
	    key = "Illegal character - Hexidecimal field!!\7";
	    break;
	  case Dir:
	  case List:
	  case Str:
	    break;	/* these cases are handled by badList */
	}
    } else {
	switch (cf->dtype) {
	  case None: break;	/* Needed to satisfy ANSI Warnings */

	  case Char:
	    key = "Illegal character - Alphanumeric field!!\007";
	    break;
	  case Int:
	  case Long:
	    key = "Illegal character - Integer field!!\007";
	    break;
	  case Double:
	  case Float:
	    key = "Illegal character - Real number field!!\007";
	    break;
	  case Hex:
	    key = "Illegal character - Hexidecimal field!!\007";
	    break;

	  case Dir:
	  case List:
	  case Str:
	    break;	/* these cases are handled by badList */
	}
    }

    if (key != NULL) {
	reset_attr(0);
	logError(0, key);
	reset_attr(cf->smode);

	putcursor(cf->row, cf->col+i);
    }
}


/* Print the error msg, put the cursor at the start */
void 
fbadlist(struct FIELDREC *cf)
{
    int i, len;
    char buf[512];
    struct CHOICEREC *cr;

    len = 0;
    buf[0] = 0;
    cr = cf->choice_list;
    for (i = 0; i < cf->num_choices && len < 140; i++, cr = cr->next) {
	strcat(buf, cr->choice);
	strcat(buf, ", ");
	len += strlen(cr->choice) + 2;
    }

    i = strlen(buf) - 2;
    buf[i++] = BEL;
    buf[i] = '\0';
    reset_attr(0);
    logError(0, "Valid choices: %s", buf);

    cf->choice_displayed = FALSE;

    putcursor(cf->row, cf->col);
    for (i = 0; i < cf->width; i++) {
	putchar(cf->fillc);
        cf->fdata[i] = cf->fillc;
    }
    cf->fdata[i] = 0;

    putcursor(cf->row, cf->col);
    reset_attr(cf->smode);
}

