/* @(#)getKchar.c	1.10 2/26/99
Copyright 2002 AOI Industries, Inc. All rights reserved.

License is granted to copy, to use, and to make and to use derivative
works for any purpose, provided that the copyright notice and this
license notice is included in all copies and any derivatives works and
in all  related documentation.  Licensor grants no other licenses expressed 
or implied and the licensee acknowleges that Licensor have no liability for 
licensee's use or for any derivative works made by licensee. 

This software is provided AS IS.

Licensor disclaim and licensee agrees that all warranties, express or
implied, including without limitation the implied warranties of
merchantability and fitness for a particular purpose.  Notwithstanding
any other provision contained herein, any liability for damages
resulting from the software or its use is expressly disclaimed,
including consequential or any other indirect damages, whether arising
in contract, tort (including negligence) or strict liability, even if
Licensor is advised of the possibility of such damages.  
*/

/*
 * Routines to scan the input stream for the next keystoke.  Many terminals
 * generate a character sequence when certain keys, notably the arrow 
 * keys, are pressed.  The 'getKchar' routine returns a single integer
 * representing the keystoke that was pressed.  Definitions for keys
 * generating sequences are in 'form.h'.  The following user routines are
 * included:
 *
 *   		Kchar_init();	* initialize buffers *
 *
 *		getKchar();	* return next keystroke *
 *
 *		ungetKchar();   * return a character *
 *
 * The routine 'initform()' must be called before 'getKchar' is used to
 * initialize the terminal dependant sequence array.

 * Exceptions:  Once a sequence has more than one character matched, the
 *              program will continue reading from stdin until a match or 
 *              mismatch happens.
 */
#ifdef __linux__
#define _POSIX_SOURCE
#endif
#define __EXTENSIONS__
#include <stdio.h>
#include <unistd.h>
#include "form.h"

#define MAX_UNGET 80
static int ungetbuff[MAX_UNGET];	/* ungetc buffer */
static int ungetcnt = 0;		/* index of last character in */

/* we'd rather use 'ungetc', but only one character pushback is guaranteed...*/
void ungetKchar(int c)
{
    if (ungetcnt < MAX_UNGET) ungetbuff[ungetcnt++] = c;
}


/* get number of characters on the unget stack */
int getKchar_cnt( void )
{
	return( ungetcnt ) ;
}

static int nextchar()
{
    int c;
    char k;

    if (ungetcnt > 0) c = ungetbuff[--ungetcnt];
    else {
	read(fileno(stdin), &k, 1);
	c = k;
    }

    return c;
}


/* Check if the character 'c' begins the sequence pointed to by 'seq'.
 * The input is read until the characters match or don't match.  If not,
 * all characters read are put back.
 *
 * If more characters are read in an attempt to match the sequence, and
 * the sequence doesn't match, the unused characters are pushed onto a
 * LIFO stack.
 */

static int testscan(int c, char *seq)	/* Check stdin for a sequence match */
{
    char buff[20];
    int i;
   
    if (!*seq) return 0;    /* sequence not defined in TERMCAP */

    i = 0;

    while (*seq == c) {
	seq++;
	if (!*seq) break;
	c = nextchar();
	buff[i++] = c;
    }

    if (*seq) {	/* No match - put characters on push down stack */
	for (--i; i >= 0; i--) ungetKchar(buff[i]);
	return 0;
    } else return 1;
}

int getKchar()		/* Get the next keystroke */
{
    int c;

    fflush(stdout);

    c = nextchar();	
    
    if (testscan(c, fm_t.kd)) return DOWN_ARROW;
    if (testscan(c, fm_t.ku)) return UP_ARROW;
    if (testscan(c, fm_t.kl)) return LEFT_ARROW;
    if (testscan(c, fm_t.kr)) return RIGHT_ARROW;
    if (testscan(c, fm_t.k1)) return PF1;
    if (testscan(c, fm_t.k2)) return PF2;
    if (testscan(c, fm_t.k3)) return PF3;
    if (testscan(c, fm_t.k4)) return PF4;
    if (testscan(c, fm_t.kh)) return HOME;
    if (testscan(c, fm_t.kc)) return CLEAR_MSG;
    if (testscan(c, fm_t.kN)) return NEXT_SCRN;
    if (testscan(c, fm_t.kP)) return PREV_SCRN;
    if (testscan(c, fm_t.help)) return HELP;

    return c;
}
