/*
 * @(#)next_choice.c	1.6 10/29/02
 *
Copyright 2002 AOI Industries, Inc. All rights reserved.

License is granted to copy, to use, and to make and to use derivative
works for any purpose, provided that the copyright notice and this
license notice is included in all copies and any derivatives works and
in all  related documentation.  Licensor grants no other licenses expressed 
or implied and the licensee acknowleges that Licensor have no liability for 
licensee's use or for any derivative works made by licensee. 

This software is provided AS IS.

Licensor disclaim and licensee agrees that all warranties, express or
implied, including without limitation the implied warranties of
merchantability and fitness for a particular purpose.  Notwithstanding
any other provision contained herein, any liability for damages
resulting from the software or its use is expressly disclaimed,
including consequential or any other indirect damages, whether arising
in contract, tort (including negligence) or strict liability, even if
Licensor is advised of the possibility of such damages.  
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "formdef.h"

/* ------------------------------------------------------------------------- */
#define MAX_CHOICE_LEN 79

static char choice[MAX_CHOICE_LEN+1];

/*
 * Function: next_choice()
 * Porpoise: To interpret a list field (after the initial '[') and return
 *           lots of exciting information about the next choice.
 * Params:   fp         - ptr to ptr to form spec (MUST be after the initial
 *                        '[').  The ptr is updated to be at the "," that
 *                        seperates each choice, or at the final bracket.
 *           HasTwoByte - current choice contains Katakana start/end
 *           len        - length of field (Not including katakana start/end
 *                        if in choice).
 * Returns:  Pointer to NULL terminated string containing next choice.
 */

char* next_choice(char **fp, int *len)
{
    char *end, *y, *begin = *fp;

    *len = 0;

    /* If not looking at a string, bail out now! */
    if (begin == NULL) return NULL;

    /* There has to be at least one item in the list field. */
    if (*begin == ']' || *begin == ')') {
        printf("\n\nnext_choice: Can not have empty list field : %s\n", *fp);
        exit(-1);
    }

    /* Skip leading blanks */
    while (*begin && *begin == ' ' ) 
        begin++;

    /* If escape char, make sure it is katakana start sequence */
    if (*begin == '\033') 
    {
        if ((*(begin+1) == '$') && 
    	    ((*(begin+2) == 'B') || (*(begin+2) == '@')))
    	{
            begin += 3;
        }
        else {
            printf("\n\nnext_choice: Illegal escape sequence: %s\n", *fp);
            exit(-1);
        }
    }

    /* scan for end of list definition */
    end = strpbrk(begin, ",])");

    if (end == NULL) {
        printf("\n\nnext_choice: List field item must have \
terminating ']': %s\n", *fp);
        exit(-1);
    }

    /* skip trailing blanks. Note nothing happens here if  katakana */
    y = end;
    while (*(y - 1) == ' ') y--;

    /* Set length */
    *len = y - begin;

    choice[0] = '\0';

    strncat(choice, begin, *len);

    *fp = end;

    return choice;
}
