/* @(#)putform.c	1.42 3/17/00
Copyright 2002 AOI Industries, Inc. All rights reserved.

License is granted to copy, to use, and to make and to use derivative
works for any purpose, provided that the copyright notice and this
license notice is included in all copies and any derivatives works and
in all  related documentation.  Licensor grants no other licenses expressed 
or implied and the licensee acknowleges that Licensor have no liability for 
licensee's use or for any derivative works made by licensee. 

This software is provided AS IS.

Licensor disclaim and licensee agrees that all warranties, express or
implied, including without limitation the implied warranties of
merchantability and fitness for a particular purpose.  Notwithstanding
any other provision contained herein, any liability for damages
resulting from the software or its use is expressly disclaimed,
including consequential or any other indirect damages, whether arising
in contract, tort (including negligence) or strict liability, even if
Licensor is advised of the possibility of such damages.  
 */

/* Forms management.  Display the form specified in the argument list, and
 * handle operator input.  The functional description is in the accompanying
 * FORM.DOC file.  The tactics are to scan the string up to an action
 * specifier, putting the text out to the screen as we go.
 *
 * As each field definition is encountered, a record is built describing the
 * field (its location, attributes, etc.).  If an attribute is defined by for
 * the field that doesn't exist, the field is filled with underscores, which
 * will be overwritten by user input.
 *
 * The length of the argument list is indefinite; bytes are peeled off as
 * needed.
 *
 * The function "initform" must be called before entry.
 *
 * Arguments:	clrscr   - Screen should be cleared before displaying the form
 *  		nocancel - If true, ^E, HOME are invalid keys
 *		field	 - Ordinal of the field to put the cursor on
 *		form	 - Character string describing the form
 *		args	 - Variable length argument list
 */
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <locale.h>

#include "form.h"
#include "formdef.h"


/*
 * Multiple field banners are located in the miscellaneous languange file.
 * They extend from columns 1-79 to clear any existing text.  Some terminals
 * scroll if we write to row 24, col80, so we avoid it by having the banners
 * padded.  If by some mysterious circumstance the banners have disappeared
 * from the database (note that by the time we get around to using the banners
 * we have already done a 'strLookup' to get the form), we will use one of the
 * ones in the following table.
 */
enum { BannerNoEscER, BannerNoEsc,  BannerNormalER, BannerNormal};

static struct { char *key;  char *val; } banner_strings[] = {
{ "frmbannoesctbl", "CR-Accept  ^U-Clear  Arrows/TAB/BS|Cursor/Next/Prev  M-Move Table  J1-Position" }, 
{ "frmbannoesc",    "CR-Accept  ^U-Clear  Arrows/TAB/BS|Cursor/Next/Prev" }, 
{ "frmbanesctbl",   "CR-Accept  ^E-Cancel  HOME  ^U-Clear  Arrows/TAB/BS  M-Move Table  J1-Position" }, 
{ "frmbanesc",      "CR-Accept  ^E-Cancel  HOME  ^U-Clear  Arrows/TAB/BS|Cursor/Next/Prev", }
};

extern int default_language;


/* Display a form, potentially waiting for pc input as well.  Return values:
	-2 - Communications Timeout
	-1 - Operator cancel
	 0 - Form completion
	 n>0 - Message available, n is ordinal of field cursor is on.

   To avoid extra communications overhead, the character attribute mode is
   not changed until a printable character is actually written to the screen.
 */
int putform(int clrscr, int nocancel, int field, const char *form_ptr,...)
{
    char *banner, *p;
    const char *fp;
    int row, col, s, ord, banner_to_get;
    struct FIELDREC *fr;
    struct FIELD_LIST fldlist;
    InvalidActions v;
    int scrmode;
    va_list ap;

    va_start(ap, form_ptr);

    fp = form_ptr;
    fldlist.head = NULL;
    fldlist.tail = NULL;
    scrmode = 0;

    row = col = ord = 1;

    /* Avoid clearing screen if it NOCLRSCREEN or it is FIELDONLY - don't
       erase 1st two lines to allow info messages to be retained.
    */
    if (clrscr == CLRSCREEN) {
	putcursor(3, 1);
	clr_eos();
    }
    /* Scan form for action definitions; write out text. */
    cursor_off();

    v.NoInputFields = 0;
    v.NoPosition = 0;

    scan_form(fp, clrscr, &scrmode, &row, &col, &fldlist, v, &ap);

    /* Put out key action banner, if any NON READ ONLY fields were defined */
    fr = 0;
    if (fldlist.head && field > 0 && clrscr != FIELDONLY) {

        /* pick a banner. */
	if (nocancel) {
            if (fr) banner_to_get = BannerNoEscER;
            else banner_to_get = BannerNoEsc;
        }
        else {
            if (fr) banner_to_get = BannerNormalER;
            else banner_to_get = BannerNormal;
        }

	banner = 0;
	p = banner_strings[banner_to_get].val;

    	putcursor(24, 1);
	clr_eol();
        s = (80 - strlen(p)) / 2;
        reset_attr(0);
        putcursor(24, s);
        fputs(p, stdout);

        if (banner) free (banner);  /* no longer need the banner */
    }  /* If we need to display a banner */

    cursor_on();
    va_end(ap);

    /* Handle keyboard input */
    s = forminput(&fldlist, nocancel, field);
    if (s == formNONE) {
	fflush(stdout);	/* guarentee display of read-only forms */
    }

    fldrelease(&fldlist);

    return s;
}
