/* @(#)scan_form.c	1.10 10/29/02
Copyright 2002 AOI Industries, Inc. All rights reserved.

License is granted to copy, to use, and to make and to use derivative
works for any purpose, provided that the copyright notice and this
license notice is included in all copies and any derivatives works and
in all  related documentation.  Licensor grants no other licenses expressed 
or implied and the licensee acknowleges that Licensor have no liability for 
licensee's use or for any derivative works made by licensee. 

This software is provided AS IS.

Licensor disclaim and licensee agrees that all warranties, express or
implied, including without limitation the implied warranties of
merchantability and fitness for a particular purpose.  Notwithstanding
any other provision contained herein, any liability for damages
resulting from the software or its use is expressly disclaimed,
including consequential or any other indirect damages, whether arising
in contract, tort (including negligence) or strict liability, even if
Licensor is advised of the possibility of such damages.  
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <form.h>
#include <formdef.h>

void scan_form(const char *fp, int clrscr, int *scrmode, int *row, int *col,
	       struct FIELD_LIST *fld_list, InvalidActions v, va_list *ap)
{
    struct FIELDREC *fr;
    int lastrow, lastcol, move_cursor, ord, rel, s;
    char *p, *q, c;

    move_cursor = FALSE;
    /*
     * scan_form is rigged to deal w/ multiple passes to scan_form.  In order
     * to allow the same here (so the two can be merged some day), we may want
     * this passed in as param 
     */
    ord = 1;
    lastrow = *row;
    lastcol = *col;
    
    while (*fp) {
	p = strpbrk(fp, "@^~%\\");
	if (!p) {
	    if (move_cursor) {
		putcursor(*row, *col);
		move_cursor = FALSE;
	    }
	    if (clrscr != FIELDONLY) {
		reset_attr(*scrmode);
		fputs(fp, stdout);
		*col += strlen(fp);
	    }
	    break;	/* all done! */
	}

	/*
	 * put out string to def point.  If we are displaying fields only, we
	 * ignore displaying the string, but we still must move the cursor.
	 */
	s = p - fp;
	if (s > 0) {
	    if (clrscr != FIELDONLY) {
		if (move_cursor) {
		    putcursor(*row, *col);
		    move_cursor = FALSE;
		}
		reset_attr(*scrmode);
		printf("%.*s", s, fp);
	    } else
		move_cursor = TRUE;

	    *col += s;
	}

	switch (*p) {

	    /* Field Definition */
	  case '%':
	    if (move_cursor) {
		putcursor(*row, *col);
		move_cursor = FALSE;
	    }

	    /* If a readonly field, deal with it specially */
	    if (*(p + 1) == '#') {
		fp = readonly(p, *scrmode, col, ap);  /* remember col is ptr */
	    } else {
		if (v.NoInputFields) {
		    printf("\n illegal syntax: Only Read-Only fields \
allowed: %s\n", p);
		    abort();
		}

		fp = fieldef(fld_list, p, &fr, *row, *col, *scrmode, ap);
		fr->ordinal = ord;

		/* If List and Directory fields have brackets */
		*col = *col + fr->width +
		    ((fr->dtype == List || fr->dtype == Dir
		      || fr->dtype == Str) ? 2:0);
	    }
	    ord++;
	    break;

	    /* Cursor placement */
	  case '@':
	    if (v.NoPosition) {
		printf("\n\nNo positions allowed: %s\n\n", p);
		abort();
	    }

	    p++;

	    /* Get absolute/relative ROW position. */
	    switch (*p) {
                case '+': rel = 1; p++; break;
                case '-': rel = -1; p++; break;
                default:  rel = 0; break;
            }

	    if (*p == '$') {
		*row = va_arg(*ap, int);
	    } else {
		s = sscanf(p, "%d;", row);  /* remember row is a ptr */
		if (s != 1) {
		    printf("\nillegal row syntax: %.15s\n",
			   (p - 1 - abs(rel)));
		    abort();
		}
	    }

	    switch (rel) {
                case 1:  *row = *row + lastrow; break;
                case -1: *row = -(*row) + lastrow; break;
                default: break;
	    }

	    p = strchr(p, ';');
	    p++;

	    /* get absolute/relative COLUMN position */
	    c = *p;
	    switch (c) {
                case '+': rel = 1; p++; break;
                case '-': rel = -1; p++; break;
                default:  rel = 0;  break;
	    }

	    switch (c) {
	      case '$':
		  *col = va_arg(*ap, int);
		  break;
	      case 'c':
		  /* centering - scan up to next '@' for string length */
		  q = p+2;	/* past "c;" */
		  s = 2;	/* since p is 2 chars before string to center*/
		  while (*q) {
		      if (*q == '@') break;
		      /* ignore attribute start/end, and char that follows */
		      if (*q == '^' || *q == '~') s += 2;
		      q++;
		  }
		  *col = (80 - (q - p - s))/2;
		  break;

	      default:
		  s = sscanf(p, "%d;", col);  /* remember col is  a ptr */
		  if (s != 1) {
		    printf("\nillegal column syntax: %.15s\n", (p-1-abs(rel)));
		    abort();
		  }
		  break;
	    }

	    switch (rel) {
                case 1:  *col = *col + lastcol; break;
                case -1: *col = -*col + lastcol; break;
                case 0:  break;
	    }

	    p = strchr(p, ';');
	    if (!p) {
		printf("\nillegal column syntax: %.15s\n", (p-1-abs(rel)));
		abort();
	    }
	    p++;

	    move_cursor = TRUE;
	    lastrow = *row;
	    lastcol = *col;
	    fp = p;
	    break;

	    /* Start Attribute */
	  case '^':
	    p++;
	    switch (*p) {
	      case '*':
		*scrmode |= BOLD;
		break;
	      case '_':
		*scrmode |= UNDERLINE;
		break;
	      case 'R':
		*scrmode |= REVERSE;
		break;
	      case 'B':
		*scrmode |= BLINK;
		break;
	      case 'S':

		/*
		 * special handling for vt100 double width mode.
		 */
		putcursor(*row, 1);
		dwide_on();
		putcursor(*row, *col);
		move_cursor = FALSE;
		break;
	      default:
		printf("\nIllegal start attribute '%c' : %.15s", *p, (p - 1));
		exit(-1);
	    }
	    fp = p + 1;
	    break;

	    /* End Attribute */
	  case '~':
	    p++;
	    switch (*p) {
	      case '*':
		*scrmode &= ~BOLD;
		break;
	      case '_':
		*scrmode &= ~UNDERLINE;
		break;
	      case 'R':
		*scrmode &= ~REVERSE;
		break;
	      case 'B':
		*scrmode &= ~BLINK;
		break;
	      case 'S':
		putcursor(*row + 1, 1);
		dwide_off();
		putcursor(*row, *col);
		move_cursor = FALSE;
		break;
	      default:
		printf("\nIllegal end attribute '%c' : %.15s", *p, (p - 1));
		abort();
	    }
	    fp = p + 1;
	    break;

	  case '\\':
	    if (clrscr != FIELDONLY) {
		if (move_cursor) {
		    putcursor(*row, *col);
		    move_cursor = FALSE;
		}
		reset_attr(*scrmode);
		putchar(*(p + 1));
	    } else
		move_cursor = FALSE;

	    (*col)++;
	    fp = p + 2;
	    break;
	}
    }
}
