/* @(#)sfax.c	1.11 10/11/00 
Copyright 2002 AOI Industries, Inc. All rights reserved.

License is granted to copy, to use, and to make and to use derivative
works for any purpose, provided that the copyright notice and this
license notice is included in all copies and any derivatives works and
in all  related documentation.  Licensor grants no other licenses expressed 
or implied and the licensee acknowleges that Licensor have no liability for 
licensee's use or for any derivative works made by licensee. 

This software is provided AS IS.

Licensor disclaim and licensee agrees that all warranties, express or
implied, including without limitation the implied warranties of
merchantability and fitness for a particular purpose.  Notwithstanding
any other provision contained herein, any liability for damages
resulting from the software or its use is expressly disclaimed,
including consequential or any other indirect damages, whether arising
in contract, tort (including negligence) or strict liability, even if
Licensor is advised of the possibility of such damages.  
*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>
#include <form.h>

#define COMMENT_LEN 60

static char getinfo[] = "\
@2;32;^*Transmit Telefax~*\
 @+2;5;^*Phone Number~*  ^R%!*29s~R\
@+2;+0;^*File Name~*     ^R%!*50s~R\
@+2;+0;^*To:~*           ^R%*39s~R\
@+2;+0;^*Company:~*      ^R%*39s~R\
@+2;+0;^*Subject:~*      ^R%*39s~R\
@+2;+0;^*Time to Send:~* ^R%*5s~R\
         Notify after each attempt: ^R%*[No,yes]~R\
@+1;+0;^*Comments:~*    ^_%*55s~_\
@+1;+0;   (use)     ^_%*55s~_\
@+1;+0;  (tab for)  ^_%*55s~_\
@+1;+0; (next line) ^_%*55s~_\
@+2;+0;1. 'To', 'Company', 'Subject', and 'Comments' are optional; 
@+1;+0;   no cover sheet will be generated if all are left blank.\
@+1;+0;2. Phone number can be an alias (/usr/local/lib/fax/alias.fax or \\~/.faxrc).";

char *fax_alias(char *name);
void strip_trailing_blanks(char *s);
void quote_quotes(char *s);
int filetype(char *name);

int
main(int ac, char **av)
{
    int s, i, notify;
    char to[80], phone[60], filename[120], *p;
    char comment[4][200], company[80], subject[80], comments[4*200], cmd[1024];
    char sendtime[6];
    char tmpfile[L_tmpnam];

    initform(0);
    to[0] = phone[0] = filename[0] = 0;
    comment[0][0] = comment[1][0] = comment[2][0] = comment[3][0] = 0;
    subject[0] = sendtime[0] = 0;
    notify = 0;

    for (;;) {
	s = putform(CLRSCREEN, CAN_ESCAPE, 1, getinfo,
		    phone, filename, to, company, subject, sendtime,
		    &notify, comment[0], comment[1], comment[2], comment[3]);

	if (s < 0) break;

	truncate_leading(phone);
	truncate_trailing(phone);

	p = phone;
	while (*p) {
	    if (isspace(*p)) break;
	    p++;
	}

	if (*p)
	    logError(0, "No blanks allowed in phone number.\007");
	else break;
    }

    clr_screen();
    fflush(stdout);
    uiSetNorm();	/* turn off 'raw' tty mode */

    if (s) {
	puts("No fax sent.");
	exit(0);
    }

    strip_trailing_blanks(to);
    strip_trailing_blanks(phone);
    strip_trailing_blanks(filename);
    strip_trailing_blanks(company);
    strip_trailing_blanks(subject);
    strip_trailing_blanks(comment[0]);
    strip_trailing_blanks(comment[1]);
    strip_trailing_blanks(comment[2]);
    strip_trailing_blanks(comment[3]);
    strip_trailing_blanks(sendtime);

    quote_quotes(to);
    quote_quotes(phone);
    quote_quotes(filename);
    quote_quotes(company);
    quote_quotes(subject);
    quote_quotes(comment[0]);
    quote_quotes(comment[1]);
    quote_quotes(comment[2]);
    quote_quotes(comment[3]);
    quote_quotes(sendtime);

    /* Wordperfect 6.0a puts a ^D in postscript output files.  Strip them
       out.
     */
    tmpnam(tmpfile);
    sprintf(cmd, "tr -d \"\\4\" < %s > %s", filename, tmpfile);
    s = system(cmd);
    if (s) {
	puts("fax not sent.");
	unlink(tmpfile);
	return -1;
    }

    strcpy(cmd, "sendfax ");

    if (sendtime[0])
	sprintf(&cmd[strlen(cmd)], " -a \"%s\" ", sendtime);

    /* Build comment string */
    comments[0] = 0;
    for (i=0; i<4; i++) {
    	if (comment[i][0]) {
	    if (i) strcat(comments, " ");	    /* word separator */
	    strcat(comments, comment[i]);
	}
    }

    if (comments[0]) {
    	sprintf(&cmd[strlen(cmd)], " -c \"%s\" ", comments);
    }

    if (subject[0]) {
	sprintf(&cmd[strlen(cmd)], " -r \"%s\" ", subject);
    }

    if (company[0]) {
	sprintf(&cmd[strlen(cmd)], " -x \"%s\" ", company);
    }

    /* check for possible alias phone number */
    if (isalpha(phone[0])) {
    	if (!(p = fax_alias(phone))) {
	    logError(0, "Phone number alias %s not found", phone);
	    unlink(tmpfile);
	    return -1;
	}
	strcpy(phone, fax_alias(phone));
    } 

    sprintf(&cmd[strlen(cmd)], " -%c ", notify?'R':'D');

    /* make sure phone number has no spaces */
    p = phone;
    while (*p && !isspace(*p)) p++;
    *p = 0;

    /* Check for no cover sheet */
    if (!to[0] && !company[0] && !subject[0] && !comments[0]) {
	strcat(cmd, " -n ");
    }

    /* added ID (version 4/not yet implemented) */
    sprintf(&cmd[strlen(cmd)], " -i \"%s\" ", filename);

    if (to[0]) {
	sprintf(&cmd[strlen(cmd)], " -d \"%s@%s\" ", to, phone);
    } else {
	sprintf(&cmd[strlen(cmd)], " -d \"%s\" ", phone);
    }

    strcat(cmd, tmpfile);

    s = system(cmd);

    unlink(tmpfile);

    if (s) {
	puts("fax not sent.");
	return -1;
    } else
	return 0;
}

/* Scan and insert '\' in front of the characters "`"\$"
 */
void 
quote_quotes(char *s)
{
    char *p;

    while ((s = strpbrk(s, "`\\$\""))) {
	p = &s[strlen(s)];
	*(p+1) = 0;
	while (p != s) {
	    *p = *(p-1);
	    p--;
	}
	*p = '\\';
	s += 2;
    }
}

void 
strip_trailing_blanks(char *s)
{
    char *p;

    p = &s[strlen(s) - 1];
    while (p >= s && isspace(*p))
	p--;

    *(p + 1) = 0;
}

/* 
   Determine file type (text or Postscript).  Help avoid sending wrong
   file as a fax.

    Returns:  -1 - error
    	       0 - text file
    	       1 - postscript
    	       2 - binary
 */

int filetype(char *name)
{
    FILE *f;
    char buff[1024], *p, *lastchar;
    int type, n, eol;

    f = fopen(name, "r");
    if (!f) {
	logError(errno, "Can't open %s", name);
	return -1;
    }

    type = 0;
    eol = 1;
    while (!type && (n = fread(buff, 1, sizeof(buff), f))) {

	p = buff;
	lastchar = &buff[n];

    	do {
	    
    	    /* Try for Postscript: %! at start of line */
    	    if (eol && *p == '%' && *(p+1) == '!') {
	    	type = 1;
	    	break;
	    }
    	    eol = 0;

    	    /* Scan up to newline */
    	    while (p != lastchar) {
	    	if (!isspace(*p) && !isprint(*p)) {
    	    	    type = 2;	    	/* binary */
		    p = lastchar;
		    break;
    	    	}

    	    	if (*p == '\n') {
		    p++;
		    eol = 1;
		    break;
		}
    	    	p++;
    	    }

	} while (p != lastchar);
    }

    fclose(f);
    return type;
}
